function [sum_viol_T1, sum_viol_T3, sum_viol_T6] = ANA_NMR_Sum_of_violations_model(folder_path,filelist_log,number_of_restrains)
%% INPUT:
% folder_path - path to the folder: str format. Should end up with '\'.
% filelist_log - list of files. Cell matlab format
% number_of_restrains - number of restrains. Scalar matlab format
%% OUTPUT:
% violations_head - two columns vector: [number upper level distance]
% distance_data   - vector of all distances.
% viol_T3, viol_T6 - 3rd and 6th order violation matrices

% From previous subroutines:
% loaded_data = [Restr. Up1  <r> <rT3> <rT6> <viol> <violT3> <violT6>];
% clc
% clear all
%% INITIALIZE VARIABLES
% number_of_restrains = 710;
violations_head = zeros(number_of_restrains,2);
% violations_head(:,1) = 0:1:number_of_restrains;
% violations_head = [Restr. Up1]
distance_data = [];
% violations_data = [ <r>_structure1 <r>_structure2 ... etc. ]
viol_T1 = zeros(number_of_restrains,1);
viol_T3 = zeros(number_of_restrains,1);
viol_T6 = zeros(number_of_restrains,1);
%% Initializing file set
% folder_path = 'D:\02_EnHD\MD_simulations_EnHD\_AMBER_100ns_2JWT\disre_pdb_central10\';
for ii = 1:length(filelist_log(:,1))
    file_path = [folder_path str2mat(filelist_log(ii,1))];
    [loaded_data_temp] = ANA_NMR_load_disre_log_model(file_path, number_of_restrains);
    str_no = ii;
    [violations_head distance_data] = ANA_NMR_organize_violations_model(violations_head, distance_data, loaded_data_temp);
end

%% CALCULATE 3rd and 6th ORDER AVERAGED VIOLATIONS
distance_ave_T1 = (mean(distance_data'))';

distance_data_T3 = distance_data.^(-3);
distance_ave_T3  = ((sum(distance_data_T3'))').^(-1/3);

distance_data_T6 = distance_data.^(-6);
distance_ave_T6  = ((sum(distance_data_T6'))').^(-1/6);

for ii = 1:length(distance_data)
  if distance_ave_T1(ii,1)-violations_head(ii,2)>0
    viol_T1(ii,1) = distance_ave_T1(ii,1)-violations_head(ii,2); 
  end
end

for ii = 1:length(distance_ave_T3)
  if distance_ave_T3(ii,1)-violations_head(ii,2)>0
    viol_T3(ii,1) = distance_ave_T3(ii,1)-violations_head(ii,2); 
  end
end

for ii = 1:length(distance_ave_T6)
  if distance_ave_T6(ii,1)-violations_head(ii,2)>0
    viol_T6(ii,1) = distance_ave_T6(ii,1)-violations_head(ii,2); 
  end
end

sum_viol_T1 = sum(viol_T1);
sum_viol_T3 = sum(viol_T3);
sum_viol_T6 = sum(viol_T6);
end
