function [chromosomes_num best_fit_evol_num spectra_in_chromosome chi_best_solution X_matrix encountering_matrix chi_evaluation_matrix] = ...
        ga_core_v107(q_vector_theory, XRS_spectra, violations_head, NMR_distance, q_vector_exp, data_exp, kk_in, number_of_species, number_of_chromosomes, mutation_multiplyer, zero_species_perc,nmr_power_averaging,savefolder_run)
%% Input description:
% q_vector_theory - the qvector for XRS theoretical spectra
% XRS_spectra     - total number of curves to fit
% violations_head - doublecolumn variable contains number of restrain and
%                   maximum distance 
% NMR_distance    - the distance between restrained pair of atoms
% q_vector_exp    - experimental scattering vector
% data_exp        - experimental data 
% kk_in                 - number of iterrations
% number_of_species     - number of species in each chromosome
% number_of_chromosomes - number of initial chromosomes
% mutation_multiplyer   - how many times we mutate initial population.
%                         Only integer number allowed.      
%% Output description:
% chromosomes_num - the number of xrs function in each generation:
%                   size: (kk+1, number_of_chromosomes, number_of_species) 
% chromosomes_best_fit_num - the best at each generation (kk+1 by number_of_species) 
% chi                 - function value for each chromosome (kk+1 by number_of_chromosomes)
% chi_best_solution   - the best function value
% X_matrix            - matrix of normalization coefficients
% encountering_matrix - matrix chows how many times each spectrum was
%                       included during mutations
% chi_evaluation_matrix - objective functions data. Raws are:
%         1. Number of the chromosome (to being evaluate)
%         2. Chi XRS data for the chromosome
%         3. Violation NMR data for the chromosome
%         4. Power of the solution (see POWER SORTING below)
%         5. Crowding distance ,(see CROWDING DISTANCE below)
%% Generation of zero species
number_of_xrs_spectra   = length(XRS_spectra(1,:)); % doublicate of number_of_spectra_total
XRS_spectra(:,end+1:end+round(zero_species_perc*length(XRS_spectra(1,:)))) = 0;
%% Initialization of variables
number_of_spectra_total = length(XRS_spectra(1,:)); 
encountering_matrix     = zeros(number_of_spectra_total,1);
background = 100;
% spectra_in_chromosome    = zeros(kk_in,1);
percentage_count = 0;
chromosomes_num  = zeros(kk_in+1,number_of_chromosomes,number_of_species);
chi_mod               = zeros(kk_in+1,number_of_chromosomes);
chi_xrs               = zeros(kk_in+1,number_of_chromosomes);
spectra_in_chromosome = zeros(kk_in+1,number_of_chromosomes);
%% Generation of the initial population
for ii = 1:number_of_chromosomes
   for jj = 1:number_of_species
      random_val = unidrnd(number_of_spectra_total,1,1); % unidrnd(a,x,y) - generates matrix x by y filled with randoms 1..a
      if random_val>number_of_xrs_spectra
          random_val = number_of_xrs_spectra+1;
      end
      chromosomes_num(1,ii,jj) = random_val;
      encountering_matrix(random_val,1) = 1+encountering_matrix(random_val,1);
      chromosomes_to_evaluate(:,jj)=XRS_spectra(:,chromosomes_num(1,ii,jj));
   end
   % checks how many different species we have in the ensemble
        chrom_unique_check=[unique(squeeze(chromosomes_num(1,ii,:))), histc(squeeze(chromosomes_num(1,ii,:)),unique(squeeze(chromosomes_num(1,ii,:))))];     
        if chrom_unique_check(end,1)>number_of_xrs_spectra
            spectra_in_chromosome(1,ii) = -1+length(chrom_unique_check);
        else
            spectra_in_chromosome(1,ii) = length(chrom_unique_check);
        end
        temp_function = mean((chromosomes_to_evaluate)');
        fiting_function = interp1(q_vector_theory,temp_function,q_vector_exp);
        fiting_function(:,2) = background;
        X_matrix_temp = pinv(fiting_function)*data_exp;
%       chi_mod(1, ii) = ...
%           sum(abs((fiting_function*squeeze(X_matrix_temp) - data_exp).^1)  );%./length(data_exp));
%       chi_xrs(1,ii) = chi_mod(1, ii);
   chromosomes_num(1,ii,:) = sort(chromosomes_num(1,ii,:));  
end
%% PREPARE FOLDER FOR SAVING PICS:
savefolder_fit_bystep = [savefolder_run,'Figures_time_evolution\'];
if exist(savefolder_fit_bystep,'dir')==0
    mkdir(savefolder_fit_bystep);
end

%% =====================%% 
%    ITTERATION CYCLE    %
%%======================%%
tic
for kk = 1:kk_in
%% Crossing
chromosomes_num_end = chromosomes_num(kk,:,:);
[chromosomes_child_num] = GA_CORE_Crossing(number_of_chromosomes, number_of_species, number_of_xrs_spectra, chromosomes_num_end);
%% Mutations 
[chromosomes_mutants_num encountering_matrix] = ...
    GA_CORE_Mutation(number_of_chromosomes, number_of_species, number_of_xrs_spectra, number_of_spectra_total, chromosomes_num_end,encountering_matrix,mutation_multiplyer);
%% ELIMINATES REPEATING ENSEMBLES    
chromosomes_to_evaluate_num = [squeeze(chromosomes_num(kk,:,:));...
         chromosomes_mutants_num; chromosomes_child_num];
number_of_chrom_to_eval = length(chromosomes_to_evaluate_num(:,1));
     
chromosomes_num_kk = chromosomes_to_evaluate_num;%squeeze(chromosomes_num(kk,:,:));
chromosomes_num_kk_unique = [];
for ii = 1:number_of_chrom_to_eval
    repeated_ii = 0;
  for jj = 1:ii-1
    if isequal(unique(chromosomes_num_kk(ii,:)),unique(chromosomes_num_kk(jj,:)))==1
      repeated_ii = 1; 
    end
  end
  if repeated_ii==0
      chromosomes_num_kk_unique(end+1,:) = chromosomes_num_kk(ii,:);
  end
end 
if length(chromosomes_num_kk_unique(:,1))<number_of_chrom_to_eval
    chromosomes_zeros = [];
   for ii = length(chromosomes_num_kk_unique(:,1))+1:number_of_chrom_to_eval
       for jj = 1:number_of_species
       chromosomes_num_kk_unique(ii,jj) = ...
           round(random('unif',1,number_of_xrs_spectra+1));
       end
   end
end
  chromosomes_to_evaluate_num = chromosomes_num_kk_unique;            
  spectra_in_chromosome_temp  = zeros(1,length(chromosomes_to_evaluate_num(:,1)));

%% FUNCTION EVALUATION
    for ii = 1:number_of_chrom_to_eval
        for jj = 1:number_of_species
          chromosomes_to_evaluate(:,jj)=XRS_spectra(:,chromosomes_to_evaluate_num(ii,jj));
%           nmr_chromosomes_to_evaluate(:,jj)=NMR_distance(:,chromosomes_to_evaluate_num(ii,jj));
        end
%% XRS fitting evaluation: linear scale
       temp_function = mean((chromosomes_to_evaluate)');
       fiting_function = interp1(q_vector_theory,temp_function,q_vector_exp);
       fiting_function(:,2) = background;
% find a normalization
       X_matrix(kk,ii,:) = pinv(fiting_function)*data_exp;
% calculation of the energy term
%        chi_xrs_out(1, ii) = ...
%            sum(abs((fiting_function*squeeze(X_matrix(kk,ii,:))) - data_exp));
%        chi_xrs_out(1, ii) = sum((((fiting_function*squeeze(X_matrix(kk,ii,:))) - data_exp).^2)./sqrt(data_exp));
%        chi_xrs_out(1, ii) = ...
%            sum(10*q_vector_exp.*abs((fiting_function*squeeze(X_matrix(kk,ii,:)) - data_exp).^1)  );%./length(data_exp));
       chi_xrs_out(1, ii) = ...
           sum(abs((fiting_function*squeeze(X_matrix(kk,ii,:)) - data_exp).^1)  );%./length(data_exp));
       
% to include NMR only !!!!!!!!!!!!!!      
%        chi_xrs_out(1, ii) = 0;
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!       
       
%% XRS fitting evaluation: log scale       
%        temp_function = mean((chromosomes_to_evaluate)');
%        fiting_function = interp1(q_vector_theory,temp_function,q_vector_exp);
%        fiting_function(:,2) = background;
% % find a normalization
%        fiting_function(:,1) = log(fiting_function(:,1));
%        X_matrix(kk,ii,:) = pinv(fiting_function)*log(data_exp);
% % calculation of the energy term
% %        chi_xrs_out(1, ii) = ...
% %            sum(abs((fiting_function*squeeze(X_matrix(kk,ii,:))) - data_exp));
% %        chi_xrs_out(1, ii) = sum((((fiting_function*squeeze(X_matrix(kk,ii,:))) - data_exp).^2)./sqrt(data_exp));
%        chi_xrs_out(1, ii) = ...
%            sum(abs((fiting_function*squeeze(X_matrix(kk,ii,:)) - log(data_exp)).^1)  );%./length(data_exp));
%        
%% NMR restrains evaluation
number_of_restrains = length(NMR_distance(:,1));
%   nmr_chromosomes_to_evaluate = zeros(number_of_restrains,1);
   nmr_chromosomes_to_evaluate = [];  
  viol_Tn = zeros(number_of_restrains,1);
        if nmr_power_averaging ~= 0
          for jj = 1:number_of_species
            if chromosomes_to_evaluate_num(ii,jj)<=number_of_xrs_spectra  
              nmr_chromosomes_to_evaluate(:,end+1)=NMR_distance(:,chromosomes_to_evaluate_num(ii,jj));
            end
          end
            distance_ave_Tn = (mean(nmr_chromosomes_to_evaluate.^(-nmr_power_averaging),2)).^(-1/nmr_power_averaging);
          for mm = 1:number_of_restrains
             if distance_ave_Tn(mm,1)-violations_head(mm,2)>0
                viol_Tn(mm,1) = distance_ave_Tn(mm,1)-violations_head(mm,2); 
             end
          end 
        end
          sum_viol_Tn(1, ii) = sum(viol_Tn);
    end
%% =============================================== %%
%   POWER SORTING + CROWDING DISTANCE ALGORITHM     %
%%=================================================%%

%% CHI_EVALUATION_MATRIX
    for ii = 1:number_of_chrom_to_eval
       if unique(chromosomes_to_evaluate_num(ii,:)) == number_of_xrs_spectra+1;
         sum_viol_Tn(1, ii) = max(sum_viol_Tn(1, :));
       end
    end 
    chi_evaluation_matrix = [1:1:number_of_chrom_to_eval;
                             chi_xrs_out(1, :);
                             sum_viol_Tn(1, :)];                              
    chi_evaluation_matrix(4,:) = number_of_chrom_to_eval;
    chi_evaluation_matrix(5,:) = number_of_chrom_to_eval;
    
   [chi_temp,temp_num] = sortrows(chi_evaluation_matrix',[2 3]);
    chi_evaluation_matrix(4,temp_num(1,1)) = -1;        
   [chi_temp,temp_num] = sortrows(chi_evaluation_matrix',[3 2]);
    chi_evaluation_matrix(4,temp_num(1,1)) = -1;        
%% CALCULATION OF POWER DOMINANCE. 
% The lower value represent better optimized solution.
% For example if the current solution is 
% dominated by 7 other solutions, its power dominance equal to 7.
% If the solution is in Pareto front, its power dominance equal to 0
% The value -1 is for extreme values, representing the border of the Pareto front.
% Those values should always be kept in the final solution (that is why value equal to -1).
    for ii = 1:number_of_chrom_to_eval
      if chi_evaluation_matrix(4,ii) == number_of_chrom_to_eval
        domination_counter = 0;
        for jj = 1:number_of_chrom_to_eval
            if chi_evaluation_matrix(2,jj)<=chi_evaluation_matrix(2,ii) && ...
               chi_evaluation_matrix(3,jj)<=chi_evaluation_matrix(3,ii) && ii~=jj
              domination_counter =  domination_counter + 1;
            end
        end
        chi_evaluation_matrix(4,ii) = domination_counter;        
      end
      for dd = 1:ii-1
        if isequal(chromosomes_to_evaluate_num(ii,:),chromosomes_to_evaluate_num(dd,:))==1
           chi_evaluation_matrix(4,ii)= number_of_chrom_to_eval; 
        end
      end
    end
% sorts the final matrix according to the power first.
% Then specify the second objective, then the first to sort.
% chi_evaluation_matrix = (sortrows(chi_evaluation_matrix',[4 3 2]))';
%% CROWDING DISTANCE
% Tries to keep diversity of generated solutions, 
% (prevents degeneration of the Pareto front).
% Crowding distance is determined in objective space.
% (As a default it is XRS vs. NMR objective space, 2D).
% The objective space is normalized, i.e. maximum value 
% for each objective is equal to unity. In such a case 
% crowding distance becomes unit-independent.
%--- objectives normalization ---%
chi_evaluation_matrix_norm(2,:) = chi_evaluation_matrix(2,:)/max(chi_evaluation_matrix(2,:)); % chi XRS data
chi_evaluation_matrix_norm(3,:) = chi_evaluation_matrix(3,:)/max(chi_evaluation_matrix(3,:)); % viol NMR data
distance_matrix = zeros(number_of_chrom_to_eval,number_of_chrom_to_eval);
crowding_matrix = zeros(number_of_chrom_to_eval,2);
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% in principle should be done for every frontier separately.
% The solution below creates less evolutionary pressure.
   for ii=1:number_of_chrom_to_eval
     for jj=1:number_of_chrom_to_eval
       for qq = 1:max(chi_evaluation_matrix(4,:))
           x1 = chi_evaluation_matrix_norm(2,ii);
           x2 = chi_evaluation_matrix_norm(2,jj);
           y1 = chi_evaluation_matrix_norm(3,ii);
           y2 = chi_evaluation_matrix_norm(3,jj);
%           distance_matrix(ii,jj) = 1/(((x1-x2)^2+(y1-y2)^2)^0.5);
          distance_matrix(ii,jj) = ((x1-x2)^2+(y1-y2)^2)^0.5;
          if ii==jj
           distance_matrix(ii,jj) = Inf;
          end
          if chi_evaluation_matrix(4,ii)~=chi_evaluation_matrix(4,jj)
           distance_matrix(ii,jj) = Inf;              
          end
       end
     end
   end
   for ii=1:number_of_chrom_to_eval
    [dm_1,dm_index_1] = min(distance_matrix(ii,:));
      distance_matrix(ii,dm_index_1) = Inf;
    [dm_2,dm_index_2] = min(distance_matrix(ii,:));
    crowding_matrix(ii,1) = (dm_1^2+dm_2^2)^0.5;
%      crowding_matrix(ii,1)=1/sum(distance_matrix(ii,:)); 
%      chi_evaluation_matrix(5,ii) = crowding_matrix(ii,1); 
   end  
   
   
% chi_evaluation_matrix = (sortrows(chi_evaluation_matrix',[4 -5 2 3]))';
% size(squeeze(X_matrix(kk,1:2,:)))
% size(chi_evaluation_matrix(:,1:2)')
chi_evaluation_matrix_temp = (sortrows([chi_evaluation_matrix', squeeze(X_matrix(kk,:,:))],[4 -5]))';
% size((sortrows([chi_evaluation_matrix(:,1:2)', squeeze(X_matrix(kk,1:2,:))],2))')
% size(chi_evaluation_matrix(:,1:2))
chi_evaluation_matrix_temp(:,1:2) = (sortrows(chi_evaluation_matrix_temp(:,1:2)',2))';
chi_evaluation_matrix = chi_evaluation_matrix_temp(1:size(chi_evaluation_matrix(:,1)),:);

%% DETERMINES NUMBER OF DIFFERENT SPECTRA IN CHROMOSOME
      for ii=1:number_of_chromosomes
        chromosomes_num(kk+1,ii,:) = chromosomes_to_evaluate_num(chi_evaluation_matrix(1,ii),:);
      end
% chi_mod(kk+1,:) = chi_xrs_out (1, 1:number_of_chromosomes);
% chi_xrs(kk+1,:) = chi_xrs_out (1, 1:number_of_chromosomes);

        for ii = 1:number_of_chromosomes
           chrom_unique_check=[unique(squeeze(chromosomes_num(kk+1,ii,:))), histc(squeeze(chromosomes_num(kk+1,ii,:)),unique(squeeze(chromosomes_num(kk+1,ii,:))))];     
           if chrom_unique_check(end,1)>number_of_xrs_spectra
              spectra_in_chromosome(kk+1,ii) = -1+length(chrom_unique_check);
           else
              spectra_in_chromosome(kk+1,ii) = length(chrom_unique_check);
           end
        end
    %
    % checks how many different species we have in the ensemble
        chrom_unique_check=[unique(chromosomes_to_evaluate_num(:,1)), histc(chromosomes_to_evaluate_num(:,1),unique(chromosomes_to_evaluate_num(:,1)))];     
        if chrom_unique_check(end,1)>number_of_xrs_spectra
            spectra_in_chromosome_temp(1,ii) = -1+length(chrom_unique_check);
        else
            spectra_in_chromosome_temp(1,ii) = length(chrom_unique_check);
        end
%% BEST FITTED CHROMOSOMES TO THE OBJECTIVE FUNCTIONS
%     chi_xrs_temp = chi_xrs_out(1,:).*power(1.000,spectra_in_chromosome_temp(1,:));
% chromosomes_num_last = squeeze(chromosomes_num(end,:,:));
% chromosomes_to_evaluate
objective_nr_1 = 1;
objective_nr_2 = 2;
   [chi_best_solution(kk),temp_num] = min(chi_evaluation_matrix(2,:));

       for jj = 1:number_of_species % generation of the best fitted spectra
          chromosomes_best_fit_obj1(:,jj) = XRS_spectra(:,chromosomes_to_evaluate_num(chi_evaluation_matrix(1,objective_nr_1),jj));
          chromosomes_best_fit_obj2(:,jj) = XRS_spectra(:,chromosomes_to_evaluate_num(chi_evaluation_matrix(1,objective_nr_2),jj));          
       end
    best_fit_evol_num(kk,:) = chromosomes_to_evaluate_num(chi_evaluation_matrix(1,objective_nr_1),:); 

%% SAVE THE PATTERN FIGURE FOR THE BEST SOLUTION
% if mod(kk,100) == 0
% fig1 = figure(1111111);
% clf
% 
% if nmr_power_averaging==0
% 
% subp2 = subplot('position',[0.1 0.1 0.4 0.3]);
% data_diff = data_exp-interp1(q_vector_theory,X_matrix(kk,temp_num,2)+X_matrix(kk,temp_num,1)*mean(chromosomes_best_fit_obj1,2),q_vector_exp);
% plot(q_vector_exp,data_diff,...
%               'LineWidth',2);
%           xlabel('q, A^{-1}','FontSize', 12);
% if kk==1
%   su2ylim = 0.2*max([-min(data_diff) max(data_diff)]);
%   su4xlim = min(min(chi_evaluation_matrix));
% end
% set(subp2,'XLim',[0 ceil(max(q_vector_exp*10))/10],...
%           'YLim',[-su2ylim su2ylim],'FontSize', 12);
% ylabel('Intensity difference') 
% 
% 
% subp1 = subplot('position',[0.1 0.4 0.4 0.58]);
% semilogy(q_vector_exp,data_exp,...
%          q_vector_exp,interp1(q_vector_theory,X_matrix(kk,chi_evaluation_matrix(1,1),2)+X_matrix(kk,chi_evaluation_matrix(1,1),1)*mean(chromosomes_best_fit_obj1,2),q_vector_exp),...
%               'LineWidth',2);    
%  ylabel('Log I'); %,'FontSize', 12
%   title(['chi = ',num2str(chi_best_solution(kk)),'    itteration: ',int2str(kk)],'FontSize',16);    
%   legend('Experiment', 'Best fited curve');
% % set(subplot(4,1,1:3),...
% %          'YLim',[min(data_exp)/1.5 1.5*max(data_exp)]);
% set(subp1,'XLim',[0 ceil(max(q_vector_exp*10))/10],...
%           'YLim',[min(data_exp)/1.5 1.5*max(data_exp)],...
%           'XTick',[],'FontSize', 12);    
%     
% % chi run
% subp3 = subplot('position',[0.89 0.1 0.1 0.88]);
% hold on
% plot(chi_evaluation_matrix(3,:),chi_evaluation_matrix(2,:),...
%      'o', 'MarkerFaceColor',[0 0 1]);
% plot(chi_evaluation_matrix(3,1:2),chi_evaluation_matrix(2,1:2),...
%      'o', 'MarkerFaceColor',[1 0 0],...
%           'MarkerEdgeColor',[1 0 0]);
% % plot(chi_evaluation_matrix(2,2:2),chi_evaluation_matrix(3,2:2),...
% %      'o', 'MarkerFaceColor',[1 1 0],...
% %           'MarkerEdgeColor',[1 1 0]);      
% hold off
% 
% %    set(subp3,'XLim',[5 max([max(chi_evaluation_matrix(2,:)) ceil(min(chi_evaluation_matrix(2,:))/5)*5])],...
%    set(subp3,'YLim',[5 1.05*chi_best_solution(1)],...
%              'XLim',[-1 1],...
%              'XTick',[],'YTick',[],'FontSize', 12);
% 
%          
% % chi convergence
% % subp4 = subplot('position',[0.58 0.1 0.40 0.46]);
% subp4 = subplot('position',[0.59 0.1 0.30 0.88]);
% 
% semilogx(chi_best_solution, 'LineWidth', 2);
% xlabel('Number of generation');
% ylabel('chi_{XRS}');
% set(subp4,'XLim',[1 kk_in],...
%           'YLim',[5 1.05*chi_best_solution(1)],...
%           'XTick',[1 10 100 1000 10000],...
%           'FontSize', 12);
% % text(10,15,...
% % 	['chi = ',num2str(chi_best_solution(kk))],...
% % 	'HorizontalAlignment','left');
% text(20,17,...
% 	['N = ',num2str(kk)],...
% 	'HorizontalAlignment','left',...
%     'FontSize',16,'FontAngle','italic');
% 
% % set(gcf, 'Position', [100, 100, 900, 500]);
% 
% % saveas(fig1,[fig_O_folder,'/Converg_m_q_w_on_',runname,'.fig']);
% % print(fig1,'-dpng','-r600',[savefolder_fit_bystep,'xrs_bfit_',kk_str,'.png']);
% % print(fig1,'-depsc','-r600',[fig_O_folder,'/Converg_m_q_w_n_',runname,'.eps']);
% 
% %  saveas(gcf,[savefolder_fit_bystep,'xrs_bfit_',kk_str,'.png']);
% else
% 
% subp1 = subplot('position',[0.1 0.4 0.4 0.58]);
% semilogy(q_vector_exp,data_exp,'-r',...
%          q_vector_exp,interp1(q_vector_theory,X_matrix(kk,objective_nr_1,2)+X_matrix(kk,objective_nr_1,1)*mean(chromosomes_best_fit_obj1,2),q_vector_exp),...
%          '-b',...
%          q_vector_exp,interp1(q_vector_theory,X_matrix(kk,objective_nr_2,2)+X_matrix(kk,objective_nr_2,1)*mean(chromosomes_best_fit_obj2,2),q_vector_exp),...
%          '-g',...
%               'LineWidth',2);    
%  ylabel('Log I'); %,'FontSize', 12
%   title(['chi = ',num2str(chi_best_solution(kk)),'    itteration: ',int2str(kk)],'FontSize',16);    
%   legend('Experiment', 'Best fitted XRS', 'Best fitted NMR');
% % set(subplot(4,1,1:3),...
% %          'YLim',[min(data_exp)/1.5 1.5*max(data_exp)]);
% set(subp1,'XLim',[0 ceil(max(q_vector_exp*10))/10],...
%           'YLim',[min(data_exp)/1.5 1.5*max(data_exp)],...
%           'XTick',[],'FontSize', 12);      
% 
%      
% subp2 = subplot('position',[0.1 0.1 0.4 0.3]);
% data_diff_1 = data_exp-interp1(q_vector_theory,X_matrix(kk,objective_nr_1,2)+X_matrix(kk,objective_nr_1,1)*mean(chromosomes_best_fit_obj1,2),q_vector_exp);
% data_diff_2 = data_exp-interp1(q_vector_theory,X_matrix(kk,objective_nr_2,2)+X_matrix(kk,objective_nr_2,1)*mean(chromosomes_best_fit_obj2,2),q_vector_exp);
% plot(q_vector_exp,data_diff_1,'-b',...
%      q_vector_exp,data_diff_2,'-g',...
%               'LineWidth',2);
%           xlabel('q, A^{-1}','FontSize', 12);
% if kk==1
%   su2ylim = 0.2*max([-min(data_diff_1) max(data_diff_1)]);
%   su4xlim = min(min(chi_evaluation_matrix));
% end
% set(subp2,'XLim',[0 ceil(max(q_vector_exp*10))/10],...
%           'YLim',[-0.05 0.05],'FontSize', 12);
% ylabel('Intensity difference') 
% 
% 
% subp4 = subplot('position',[0.59 0.1 0.39 0.88]);
% hold on
% plot(chi_evaluation_matrix(3,:),chi_evaluation_matrix(2,:),...
%      'o', 'MarkerFaceColor',[0 0 1]);
% plot(chi_evaluation_matrix(3,1:2),chi_evaluation_matrix(2,1:2),...
%      'o', 'MarkerFaceColor',[1 0 0],...
%           'MarkerEdgeColor',[1 0 0]);
% hold off
% % plot(chi_best_solution, 'LineWidth', 2);
% xlabel('chi_{NMR}, nm');
% ylabel('chi_{XRS}, intensity units');
% set(subp4,'XLim',[0 2],...
%           'YLim',[0 10],...
%           'XTick',[0 .5 1 1.5 2 2.5 3 4 5 6 7 8 9 10 15 20 25 30],...
%           'YTick',[0 1 2 3 4 5 6 7 8 9 10 15 20 25 30],...          
%           'FontSize', 12);
% % text(10,15,...
% % 	['chi = ',num2str(chi_best_solution(kk))],...
% % 	'HorizontalAlignment','left');
% text(6,22,...
% 	['N = ',num2str(kk)],...
% 	'HorizontalAlignment','left',...
%     'FontSize',16,'FontAngle','italic');
%     
% end
% 
% 
% set(gcf, 'PaperUnits', 'centimeters');
% set(gcf, 'PaperPosition', [0 0 18 12]); %[x1 x2 y1 y2]
% 
%   kk_str = int2str(kk);
%   for hh = 1:5-max(size(kk_str))
%       kk_str = ['0' kk_str];
%   end
% 
% % if kk<=10
% %   for qqq = 1:25
% %     print(fig1,'-dpng','-r300',[savefolder_fit_bystep,'xrs_bfit_',kk_str,int2str(qqq),'.png']);
% %   end      
% % elseif kk<=50
% %   for qqq = 1:6
% %     print(fig1,'-dpng','-r300',[savefolder_fit_bystep,'xrs_bfit_',kk_str,int2str(qqq),'.png']);
% %   end   
% % elseif kk<=100
% %   for qqq = 1:4
% %     print(fig1,'-dpng','-r300',[savefolder_fit_bystep,'xrs_bfit_',kk_str,int2str(qqq),'.png']);
% %   end   
% % elseif kk<=400
% %   for qqq = 1:400/kk    
% %     print(fig1,'-dpng','-r300',[savefolder_fit_bystep,'xrs_bfit_',kk_str,int2str(qqq),'.png']);
% %   end
% % else
% %     print(fig1,'-dpng','-r300',[savefolder_fit_bystep,'xrs_bfit_',kk_str,'.png']);    
% % end
% 
% 
% %     print(fig1,'-dpng','-r300',[savefolder_fit_bystep,'xrs_bfit_',kk_str,'.png']);    
% 
% end
%% Visualization of the progress
   if 100*kk/kk_in >= percentage_count+1
      display([int2str(percentage_count),'%, ',int2str(kk_in*toc/kk-toc), ' sec to complete' ]) 
      percentage_count = round(100*kk/kk_in);
   end
end

end